const sanitize = require('./sanitize'),
    CircularJSON = require('circular-json'),

    /**
     * @method saniziteMessages
     * @description it iterates over the messages to sanitize
     * @param {Iterable=} messages
     */
    saniziteMessages = function (messages) {
        if (Array.isArray(messages)) {
            return messages.map(sanitize);
        }
        return messages;
    };

module.exports = {
    /**
     * @description It format the message it receives for file
     * @param {[Object={}]} info
     * @return {String}
     */
    file (info = {}) {
        let messages = '';
        try {
            messages = CircularJSON.stringify(saniziteMessages(info.messages));
        }
        catch (e) {
            console.error(e);
        }
        return `[${info.sessionId}][${info.timestamp}][${info.origin}][${info.level}]${messages}`;
    },

    /**
     * @description It format the message it receives, for console
     * @param {[Object={}]} info
     * @return {*}
     */
    consoleFormat (info = {}) {
        let messages = '';
        try {
            messages = CircularJSON.stringify(saniziteMessages(info.messages));
        }
        catch (e) {
            console.error(e);
        }
        return `${info.timestamp} ${info.origin} ${info.level} ${messages}`;
    }
};
